/* Copyright 2009 Fog Creek Software, Inc. */

using System;
using System.Collections.Generic;
using System.Text;
using System.Web;
using System.Data;

/* FogBugz namespaces-- make sure you add the neccesary assembly references to
 * the following DLL files contained in C:\Program Files\FogBugz\Website\bin\ 
 * FogBugz.dll, FogCreek.Plugins.dll, FogCreek.Plugins.InterfaceEvents.dll     */
using FogCreek.FogBugz.Plugins;
using FogCreek.FogBugz.Plugins.Api;
using FogCreek.FogBugz.Plugins.Entity;
using FogCreek.FogBugz.Plugins.Interfaces;
using FogCreek.FogBugz;
using FogCreek.FogBugz.UI;
using FogCreek.FogBugz.UI.Dialog;
using FogCreek.FogBugz.Database;

namespace IPluginCVSJoin_Commit_Example
{
    /* Class Declaration: Inherit from Plugin, expose IPluginCVSJoin, IPluginCVSDisplay,
     * IPluginCVSCommit */
    public class IPluginCVSJoin_Commit_Example : Plugin, IPluginCVSJoin, IPluginCVSCommit, IPluginDatabase
    {
        /* The plugin Id is a required argument for CCVS.SetPluginField and 
         * CCVS.GetPluginField */

        protected const string sPluginId =
            "IPluginCVSJoin_Commit_Example@fogcreek.com";

        /* We'll need this member variable to store the "CodeName" value in the 
        * pre-commit phase, so that if we need to roll back we can replace the original
        * value. */

        int preCommitAwesomenessLevel = 1;

        /* Constructor: We'll just initialize the inherited Plugin class, which 
         * takes the passed instance of CPluginApi and sets its "api" member variable. */
        public IPluginCVSJoin_Commit_Example(CPluginApi api)
            : base(api)
        {
        }

        #region IPluginCVSJoin Members

        public string[] CVSJoinTables()
        {
            /* All tables specified here must have an integer ixCVS column so FogBugz can
             * perform the necessary join. */

            return new string[] { "CVSAwesomeness" };
        }

        #endregion


        #region IPluginCVSCommit Members

        public void CVSCommitAfter(CCvs CVS)
        {

            api.Notifications.AddMessage(@"""Awesomeness"" plugin CVSCommitAfter called");
        }

        public bool CVSCommitBefore(CCvs CVS)
        {

            /* Set a member variable with the previous value in case of rollback */
            preCommitAwesomenessLevel = Convert.ToInt32(
                CVS.GetPluginField(sPluginId, "ixAwesomeness"));

            /* Assign the CVS an "Extreme" level of awesomeness */
            CVS.SetPluginField(sPluginId, "ixAwesomeness", 5);

            api.Notifications.AddMessage(
                @"""Awesomeness"" plugin returning TRUE from CVSCommitBefore");

            return true;
        }

        public void CVSCommitRollback(CCvs CVS)
        {
            /* Roll back to pre-commit value */
            CVS.SetPluginField(sPluginId, "ixAwesomeness", preCommitAwesomenessLevel);

            api.Notifications.AddMessage(@"""Awesomeness"" plugin CVSCommitRollback called");
        }

        #endregion

        #region IPluginDatabase Members

        public CTable[] DatabaseSchema()
        {
            /* for this plugin, we'll need a table repesenting the possible levels of
             * Awesomeness, and a CVS-to-Awesomeness table to allow for a join */

            CTable Awesomeness = api.Database.NewTable(api.Database.PluginTableName("Awesomeness"));
            Awesomeness.sDesc = "Levels of Awesomeness";
            Awesomeness.AddAutoIncrementPrimaryKey("ixAwesomeness");
            Awesomeness.AddTextColumn("sAwesomenessLevel", "Level of Awesomeness");

            CTable CVSAwesomeness = api.Database.NewTable(api.Database.PluginTableName("CVSAwesomeness"));
            CVSAwesomeness.sDesc = "Assigns CVSs to levels of Awesomeness";
            CVSAwesomeness.AddAutoIncrementPrimaryKey("ixCVSAwesomeness");
            CVSAwesomeness.AddIntColumn("ixCVS", true, 1);
            CVSAwesomeness.AddIntColumn("ixAwesomeness", true, 2);

            return new CTable[] { Awesomeness, CVSAwesomeness };

        }

        public int DatabaseSchemaVersion()
        {
            return 1;
        }

        public void DatabaseUpgradeAfter(int ixVersionFrom, int ixVersionTo,
            CDatabaseUpgradeApi apiUpgrade)
        {
            /* Create 5 different awesomeness levels. Note that we'll have a "None 
             * specified" level, which our plugin will treat the same as NULL. */

            string[] AwesomenessLevels = 
                { "-- None specified --", "Low", "Moderate", "High", "Extreme" };

            for (int i = 0; i < AwesomenessLevels.Length; i++)
            {
                CInsertQuery iq = api.Database.NewInsertQuery(
                    api.Database.PluginTableName("Awesomeness"));

                iq.InsertString("sAwesomenessLevel", AwesomenessLevels[i]);
                iq.Execute();
            }
        }

        public void DatabaseUpgradeBefore(int ixVersionFrom, int ixVersionTo,
            CDatabaseUpgradeApi apiUpgrade)
        {

        }

        #endregion
    }

}